
#
# GENERATED WITH PDL::PP! Don't modify!
#
package PDL::Graphics::Karma;

@EXPORT_OK  = qw(  kim krgb kstarted kcur  kvis  kview  koords  kpvslice  krenzo  kshell  xray  kslice_3d PDL::PP ksend2D PDL::PP ksend3D PDL::PP krgb_private PDL::PP krgb3d_private PDL::PP koverlay );
%EXPORT_TAGS = (Func=>[@EXPORT_OK]);

use PDL::Core;
use PDL::Exporter;
use DynaLoader;



   
   @ISA    = ( 'PDL::Exporter','DynaLoader' );
   push @PDL::Core::PP, __PACKAGE__;
   bootstrap PDL::Graphics::Karma ;





=head1 NAME

PDL::Graphics::Karma - interface to Karma visualisation applications

=head1 DESCRIPTION

Can send PDL 2D/3D data to kview, xray, kslice_3d, etc...

Data is transferred using shared memory when available on
the OS (and segments big enough - e.g. Linux but not
Solaris unless tuned) so ought to be very fast.

You can say

  perldl> kim $a, {App=>'xray'}

to send to a specific viewer and/or

  perldl> kim $a, {BB=>[0,50,-100,100]}

to specify the bounding box in world coordinates (here for a 2D image)
or just

  perldl> kim $a

to reuse the last viewer.

You can start the viewers from PDL.

For further info about Karma see http://www.atnf.csiro.au/karma.
The binary distribution can be downloaded from
http://www.atnf.csiro.au/karma/ftp.html.


=head1 SYNOPSIS

 use PDL::Karma;
 kview;
 kim $data;

=head1 FUNCTIONS

=cut







=head1 FUNCTIONS



=cut




use vars qw($LASTAPP);
$LASTAPP = 'kview'; # Default application



use PDL::Options;

# kim - send image data to karma app

=head2 kim

=for ref

Sends piddle data array to an external Karma application for viewing

=for usage

 kim($pdl, [$karma-app, $lut])

Sends $pdl data to Karma application viewer. Remembers the
last one used [default: kview].

=cut


sub kim {
  barf('Usage: kim $pdl [,{App => $karma-app, LUT => $lut, BB => $bb}]')
     if $#_==-1;
  my $pdl = shift;
  my $hash = shift;
  my ($app,$inds);
  $app = $LASTAPP unless ($app = delete($hash->{App}));
  my ($lut,$haslut) = (PDL->zeroes(PDL::byte,1,1),0);
  if (defined($hash->{LUT})) {
	$lut = $hash->{LUT};
	$haslut = 1;
  }

  if ($pdl->getndims == 3) {
  $inds = [0,$pdl->getdim(0)-1,0,$pdl->getdim(1)-1,0,
	$pdl->getdim(2)-1] unless $inds=delete($hash->{BB});
      ksend3D ($pdl, $lut,$haslut,$app,$inds);
   }
  else {
      $inds = [0,$pdl->getdim(0)-1,0,$pdl->getdim(1)-1]
         unless $inds=delete($hash->{BB});
      ksend2D ($pdl, $lut, $haslut,$app,$inds,$PDL::verbose);
  }
  $LASTAPP = $app;
}

=head2 kstarted

=for usage

kstarted([$karma-app])

=for ref

Tests if a Karma application is running.

It tries to connect to the karma application, returns 1 on
success, 0 otherwise

Can be used to check if a karma application has already been
started, e.g.

  xray unless kstarted 'xray';

=cut

sub kstarted {
  barf('Usage: kstarted [$karma-app]')
     if $#_>0;
  my $app = $#_ > -1 ? shift : $LASTAPP;
  $LASTAPP = $app;
  return kconnect($app);
}


=head2 krgb

=for usage

krgb($lut, [$karma-app])

=for ref

Sends RGB image to an external Karma application for viewing


Does not change current default viewer.

=cut

sub krgb {
  barf('Usage: krgb($lut, [$karma-app])') if $#_==-1;
  my @args = @_;
  push @args, $LASTAPP if $#_==0; 
  barf "must be [3,..] rgb piddle" unless $args[0]->getdim(0) == 3;
  if ($args[0]->getndims <= 3) { krgb_private(@args) } else {
	krgb3d_private(@args)}
};




=head2 kvis()

=for ref

Starts external Karma application kvis

=for usage

kvis([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub kvis {  # Start kvis
   if( !($pid = fork)) {	# error or child
      exec("kvis", @_) if defined $pid;
      die "Can't start kvis: $!
";
   }
   $LASTAPP = "kvis";
   return $pid;
}




=head2 kview()

=for ref

Starts external Karma application kview

=for usage

kview([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub kview {  # Start kview
   if( !($pid = fork)) {	# error or child
      exec("kview", @_) if defined $pid;
      die "Can't start kview: $!
";
   }
   $LASTAPP = "kview";
   return $pid;
}




=head2 koords()

=for ref

Starts external Karma application koords

=for usage

koords([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub koords {  # Start koords
   if( !($pid = fork)) {	# error or child
      exec("koords", @_) if defined $pid;
      die "Can't start koords: $!
";
   }
   $LASTAPP = "koords";
   return $pid;
}




=head2 kpvslice()

=for ref

Starts external Karma application kpvslice

=for usage

kpvslice([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub kpvslice {  # Start kpvslice
   if( !($pid = fork)) {	# error or child
      exec("kpvslice", @_) if defined $pid;
      die "Can't start kpvslice: $!
";
   }
   $LASTAPP = "kpvslice";
   return $pid;
}




=head2 krenzo()

=for ref

Starts external Karma application krenzo

=for usage

krenzo([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub krenzo {  # Start krenzo
   if( !($pid = fork)) {	# error or child
      exec("krenzo", @_) if defined $pid;
      die "Can't start krenzo: $!
";
   }
   $LASTAPP = "krenzo";
   return $pid;
}




=head2 kshell()

=for ref

Starts external Karma application kshell

=for usage

kshell([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub kshell {  # Start kshell
   if( !($pid = fork)) {	# error or child
      exec("kshell", @_) if defined $pid;
      die "Can't start kshell: $!
";
   }
   $LASTAPP = "kshell";
   return $pid;
}




=head2 xray()

=for ref

Starts external Karma application xray

=for usage

xray([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub xray {  # Start xray
   if( !($pid = fork)) {	# error or child
      exec("xray", @_) if defined $pid;
      die "Can't start xray: $!
";
   }
   $LASTAPP = "xray";
   return $pid;
}




=head2 kslice_3d()

=for ref

Starts external Karma application kslice_3d

=for usage

kslice_3d([OPTIONS])

=for example

 perldl> kview (-num_col => 42)
 perldl> xray

=cut

sub kslice_3d {  # Start kslice_3d
   if( !($pid = fork)) {	# error or child
      exec("kslice_3d", @_) if defined $pid;
      die "Can't start kslice_3d: $!
";
   }
   $LASTAPP = "kslice_3d";
   return $pid;
}





*ksend2D = \&PDL::ksend2D;




*ksend3D = \&PDL::ksend3D;




*krgb_private = \&PDL::krgb_private;




*krgb3d_private = \&PDL::krgb3d_private;




=head2 koverlay

=for sig

  Signature: (x(); y(); r(); ell(); PA(); fill(); int id(); char* karma_app; char* colour; int coordtype; int dotext)


=head2 koverlay

=for ref

Overlay graphics markers on a Karma application (e.g. kview)

=for usage

koverlay $x, $y, {Options...}

Currently the only markers supported are ellipses. The default
is a circle of radius 10 units,

=for example

$x = 10*xvals(10);
koverlay $x, sqrt($x), {Radius=>$x/3, Colour=>'green', App=>'kpolar'}

=for options

  Radius - [piddle] specify radius of ellipses (major axis if ellipse). Default = 10 units.
  Ellip  - [piddle] specify ellipticity of ellipses. Default = 0 i.e. circle.
  PA     - [piddle] specify principle axis (degrees rotation anticlockwise
                    from the Y axis). Default.
  ID     - [piddle] Numeric integer id labels to apply.
  Colour - [string] Colour name for overlay (e.g.  'red'). Default = 'blue'
  App    - [string] name of Karma app to send too
  Fill   - [piddle] whether outlines are filled (0 or 1). (Note filled,
                    ellipses are not yet available in Karma).
  Coords - [string] "World" or "Pixel" - type of coordinates for x/y/r.
                    Note pixel implementation rounds to nearest pixel due
		    to Karma overlays not supporting proper IMAGE_PIXEL
		    coordinates.





=cut





sub PDL::koverlay {
   my $hash = ref($_[-1]) eq "HASH" ? pop @_ : {};
   barf("Usage: koverlay: \$x, \$y, {Options...} \n") if @_ != 2;
   my($x,$y) = @_;
   $hash = {iparse({App => $LASTAPP,
                    Colour => "blue",
                    Radius => 10,
                    Ellipse => 0,
                    PA => 0,
                    Fill => 0,
                    ID => undef,
                    Coords => "WORLD"},$hash)};
   my ($app,$col,$rad,$ell,$PA,$fill,$id,$dotext);
   $app = $hash->{App};
   $col = $hash->{Colour};
   $rad = $hash->{Radius};
   $ell = $hash->{Ellipse};
   $PA  = $hash->{PA};
   $fill = $hash->{Fill};
   $dotext = 1;
   unless (defined ($id = $hash->{ID})) {
     $id = pdl(0); $dotext = 0;
   }

   my $supported = {"WORLD"=>1, "PIXEL"=>2};
   my $type=1;
   $type = $supported->{uc($hash->{Coords})} if defined $hash->{Coords};
   barf "koverlay: Unsupported coordinate type" unless $type;

   &PDL::_koverlay_int($x,$y,$rad,$ell,$PA,$fill,$id,$app,$col,$type,$dotext);
   $LASTAPP = $app;

}


*koverlay = \&PDL::koverlay;


=head2 kcur

=for ref

Return cursor position from a Karma application (e.g. kview/xray)

=for usage

 ($x,$y) = kcur($ch, {App=>'karma-app',Coords=>"World|Pixel"})

This function connects to a Karma application and returns
the ($x,$y) position and the character typed ($ch)
by the user. By default world coordinates are returned.

=for example

  print kcur {App=>"kview", Coords=>"World"}

=cut

sub kcur {
    my $hash = pop if ref($_[$#_]) eq "HASH";
    my $supported = {"WORLD"=>1, "PIXEL"=>2};
    my $type=1;
    $type = $supported->{uc($hash->{Coords})} if defined $hash->{Coords};
    barf "kcur: Unsupported coordinate type" unless $type;
    my $app;
    $app = $LASTAPP unless ($app = delete($hash->{App}));
    my ($x,$y,$ch) = &_kcur_int($app,$type);
    $_[0] = $ch; # Pass this back in args
    $LASTAPP = $app;
    return ($x,$y);
}



;



=head1 AUTHORS

Copyright (C)  1997-2001 Christian Soeller, Karl Glazebrook. Reproducing
documentation from the pdl distribution in any way that does not include a
statement telling who the original authors are is forbidden.  Reproducing
and/or distributing the documentation  in any  form that  alters the text is
forbidden. This module is free software and can be distributed under the
same terms as PDL itself.

=cut





# Exit with OK status

1;

		   